/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Paste and Go 2.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Holio Lin <gaghiel@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);
var gAccelKeyCode = prefs.getIntPref("ui.key.accelKey");
function nameFromCode(code) {
    switch(code) {
    case 16:
        return 'shift';
    case 17:
        return 'control';
    case 18:
        return 'alt';
    case 224:
        return 'meta';
    //case gAccelKeyCode:
    //    return 'accel';
    default:
        return null; // if unsupported
    }
}
var gAccelKeyString = nameFromCode(gAccelKeyCode);

var MenuRoll = {
    menuRolling: false,
    currentItemIndex: 0,
    searchbar: null,
    popup: null,
    //engines: null, // dynamic
    items: null,
    rollKey: null,
    rollModifiers: null,
    delayMS: 0,
    isBeginWithCurrent: false,
    isSwitchAfterRolled: false,
    textbox: null,
    // searchService: null,
    rdfService: null, // for 1.5 use
    internetSearchDS: null, // for 1.5 use

    startup: function(pgPref) {
        this.searchbar = pgPref.pgSearchbar; // searchbar;

        // for version compatible
        if ('_popup' in this.searchbar) { // if 2.0+
            this.popup = this.searchbar._popup;
            this.items = this.popup.childNodes;
        } else { // if 1.5
            this.popup = document.getAnonymousElementByAttribute(this.searchbar,
                    'anonid', 'searchbar-popup');
            var nodes = this.popup.childNodes;

            // in 1.5, popup.childNodes[0] here is a 'xul:template', so skip
            this.items = [];
            var n = nodes.length;
            for (var i = 1; i < n; i++)
                this.items.push(nodes[i]);
        }

        //this.engines = searchbar._engines;
        this.rollKey = pgPref.pgSearchKey;
        this.rollModifiers = pgPref.pgSearchAccelKeys;

        window.addEventListener("keypress", this.onceKeyPress, true);
        window.addEventListener("keyup", this.onceKeyUp, true);

        this.popup.addEventListener("popuphidden", this.onceMenuHidden, false);

        this.popup.addEventListener("DOMMenuItemActive", this.onceItemActive, true);
        this.popup.addEventListener("DOMMenuItemInactive", this.onceItemInactive, true);

        this.delayMS = pgPref.pgSearchEnginesPopupDelayMS;
        this.isBeginWithCurrent = pgPref.pgRollBeginWithCurrent;
        this.isSwitchAfterRolled = pgPref.pgSwitchAfterRolled;
        this.textbox = pgPref.pgSearchTextbox;

        this.onceMenuHidden(null); // do this for initial
    },

    shutdown: function() {
        window.removeEventListener("keypress", this.onceKeyPress, true);
        window.removeEventListener("keyup", this.onceKeyUp, true);
    },

    getEngines: function() { // dynamic
        // for version compatible
        if ('_engines' in this.searchbar) // 2.0 - 3.0
            return this.searchbar._engines;
        else { // searchbar of 1.5 have no field '_engines'
            if (!this.rdfService) { // lazy initial
                this.rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                      .getService(Components.interfaces.nsIRDFService);
                this.internetSearchDS = this.rdfService.GetDataSource("rdf:internetsearch");
            }

            var engines = [];

            const kNC_Root  = this.rdfService.GetResource("NC:SearchEngineRoot");
            const kNC_Child = this.rdfService.GetResource("http://home.netscape.com/NC-rdf#child");
            const kNC_Name  = this.rdfService.GetResource("http://home.netscape.com/NC-rdf#Name");

            var arcs = this.internetSearchDS.GetTargets(kNC_Root, kNC_Child, true);
            while (arcs.hasMoreElements()) {
                var rEngine = arcs.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
                engines.push(rEngine.Value);
            }

            return engines;
            // clone code from mozilla-1.5/browser/base/content/search.xml
        }
    },

    setCurrentEngine: function(target) {
        if ('currentEngine' in this.searchbar) // 2.0+
            this.searchbar.currentEngine = target;
        else { // 1.5
            this.textbox.currentEngine = target;
        }
    },

    getCurrentEngine: function() {
        var cengine = this.searchbar.currentEngine;
        if (cengine) // 2.0+
            return cengine;
        else { // 1.5
            return this.textbox.currentEngine;
        }
    },

    onceItemActive: function(event) {
        if (MenuRoll.menuRolling)
            if (event.target != MenuRoll.items[MenuRoll.currentItemIndex])
                event.target.setAttribute("_moz-menuactive", "false"); // anti default interaction
    },

    onceItemInactive: function(event) {
        if (MenuRoll.menuRolling)
            if (event.target == MenuRoll.items[MenuRoll.currentItemIndex])
                event.target.setAttribute("_moz-menuactive", "true"); // anti default interaction
    },

    onceKeyPress: function(event) {
        // if ((String.fromCharCode(event.charCode).toUpperCase() == MenuRoll.rollKey.toUpperCase()) 
        if (keyMatches(MenuRoll.rollModifiers, MenuRoll.rollKey, event)) {
            MenuRoll.roll();

            event.stopPropagation();
            event.preventDefault();
        }
    },

    roll: function() {
        if (MenuRoll.currentItemIndex < 0) { // isBeginWithCurrent && just initial
            MenuRoll.currentItemIndex = indexOf(MenuRoll.getEngines(), MenuRoll.getCurrentEngine());
        }

        if (MenuRoll.menuRolling) {
            MenuRoll.items[MenuRoll.currentItemIndex].setAttribute("_moz-menuactive", "false");
            MenuRoll.currentItemIndex = (MenuRoll.currentItemIndex + 1) % MenuRoll.getEngines().length; // step
        } else {
            MenuRoll.menuRolling = true;

            if (MenuRoll.delayMS == 0)
                MenuRoll.popup.click();
            else {
                window.setTimeout( function() {
                    if (MenuRoll.menuRolling) {  // synchronization
                        if ('currentEngine' in MenuRoll.searchbar) // 2.0+
                            MenuRoll.popup.click(); 
                        else { // 1.5
                            var keep = MenuRoll.getCurrentEngine();
                            MenuRoll.popup.click(); // show
                            // fixed(bug): under 1.5 click() of popup will clean currentEngine
                            MenuRoll.setCurrentEngine(keep); // this will close popup
                            // currentEngine's setter handles the rest
                        }
                    }
                }, MenuRoll.delayMS);
            }
        }
        MenuRoll.items[MenuRoll.currentItemIndex].setAttribute("_moz-menuactive", "true");

    },

    onceMenuHidden: function(event) { // reset state
        var cIndex = MenuRoll.currentItemIndex;
        if (cIndex >= 0)
            MenuRoll.items[cIndex].setAttribute("_moz-menuactive", "false");

        MenuRoll.menuRolling = false;

        if (MenuRoll.isBeginWithCurrent) {
            // MenuRoll.currentItemIndex = MenuRoll.getEngines().indexOf(MenuRoll.getCurrentEngine());
            /*var ceg = MenuRoll.getCurrentEngine();
            var engines = MenuRoll.getEngines(); // null here

            for (var i = engines.length - 1; i >= 0; --i)
                if (engines[i] == ceg)
                    MenuRoll.currentItemIndex = i;*/
            MenuRoll.currentItemIndex = -1;
        } else
            MenuRoll.currentItemIndex = 0;
    },

    onceKeyUp: function(event) {
        //if (MenuRoll.menuRolling && event.keyCode == event.DOM_VK_CONTROL) {// && !event.altKey && !event.metaKey)) {
        if (MenuRoll.menuRolling && isKeyOneOfAccel(MenuRoll.rollModifiers, event.keyCode)) { // one of accel key up
            MenuRoll.menuRolling = false; // avoid synchronization failedd
            
            var rolled = MenuRoll.getEngines()[MenuRoll.currentItemIndex];
            searchByEngine(rolled);
            
            if (MenuRoll.isSwitchAfterRolled)
                MenuRoll.setCurrentEngine(rolled); // setter of currentEngine of 2.0+ will close popup

            MenuRoll.popup.hidePopup();   // won't invoke onceMenuHidden
            MenuRoll.onceMenuHidden();
        }
    },

} // end of MenuRoll

function indexOf(array, object) {
    if (! 'length' in array)
        return -2;

    for(var i = array.length - 1; i >= 0; i--)
        if (array[i] == object)
            return i;

    return -1;
}

function keyMatches(modifiers, key, event) {
    var matches = (String.fromCharCode(event.charCode).toUpperCase() == key.toUpperCase());

    var modifiers = modifiers.replace('accel', gAccelKeyString);
    function syntropy(str, target) {
        return ((modifiers.search(str) != -1) // str  in modifiers
                == target);                   // attr in event
    }
    matches &= syntropy('shift', event.shiftKey);
    matches &= syntropy('alt', event.altKey);
    matches &= syntropy('meta', event.metaKey);
    matches &= syntropy('control', event.ctrlKey);

    return matches;
}

function isKeyOneOfAccel(accel, keyCode) {
    accel = accel.replace('accel', gAccelKeyString);
    var name = nameFromCode(keyCode);
    if (name) {
        return accel.search(name) != -1;
    } else {
        return false;
    }
}
